(******************************************
   CS520 hw4
*******************************************)

signature EVAL = 
sig
   type exp = INT of int
            | VAR of id
            | FN of id * exp
            | FX of id * exp
            | APP of exp * exp
            | LET of decl * exp
            | IF of exp * exp * exp
            | BOP of bop * exp * exp
            | READ
            | WRITE of exp
    and id = string
    and decl = id * exp
    and bop = ADD | SUB | MUL | DIV | EQ | LT | GT | AND | OR

   type env

   val emptyEnv: env
   val eval: env * exp -> unit
end

structure M: EVAL = 
struct
	open Format 
  	
   	type exp = INT of int
             | VAR of id
             | FN of id * exp
             | FX of id * exp
             | APP of exp * exp
             | LET of decl * exp
             | IF of exp * exp * exp
             | BOP of bop * exp * exp
             | READ
             | WRITE of exp
     and id = string
     and decl = id * exp
     and bop = ADD | SUB | MUL | DIV | EQ | LT | GT | AND | OR

		
		type env = int   		// modify this definition 
		val emptyEnv = 0 		// modify this definition 

    exception PrettyPrintNonsense
    fun printExpr n = fn							      	
        (INT m) =>  nl(); spaces n; print_int m
      | (VAR s) =>   nl(); spaces n; print_string s
      | (FN(x,e)) => nl(); spaces n; print_string ("fn "^x^" => ");
                   printExpr (n+1) e
      | (FX(x,e)) => nl(); spaces n; print_string ("fix "^x^" ");
                     printExpr (n+1) e
      | (APP(e,e')) => nl(); spaces n; print_string "app";
                       printExpr (n+1) e; printExpr (n+1) e'
      | (IF(e,e',e'')) => nl(); spaces n; print_string "if";
                        printExpr (n+1) e;
                        nl(); spaces n; print_string "then";
                        printExpr (n+1) e';
                        nl(); spaces n; print_string "else";
                        printExpr (n+1) e''
      | READ => nl(); spaces n; print_string ("read ")
      | (WRITE e) => nl(); spaces n; print_string "write";
                   printExpr (n+1) e
      | (LET(d,s)) => nl(); spaces n; print_string "let";
                    printDecl (n+1) d;
                    nl(); spaces n; print_string "in";
                    printExpr (n+1) s;
                    nl(); spaces n; print_string "end"
      | (BOP(ADD,e,e')) => printBExpr n "+" e e'
      | (BOP(SUB,e,e')) => printBExpr n "-" e e'
      | (BOP(MUL,e,e')) => printBExpr n "*" e e'
      | (BOP(DIV,e,e')) => printBExpr n "/" e e'
      | (BOP(LT,e,e'))  => printBExpr n "<" e e'
      | (BOP(GT,e,e'))  => printBExpr n ">" e e'
      | (BOP(EQ,e,e'))  => printBExpr n "=" e e'
      | (BOP(AND,e,e')) => printBExpr n "and" e e'
      | (BOP(OR,e,e'))  => printBExpr n "or" e e'

    and printBExpr n s e e' =
        (nl(); spaces n; print_string s;
         printExpr (n+1) e;
         printExpr (n+1) e')
    and printDecl n = fn
        (x,e) =>
               nl(); spaces n; print_string (x^" = ");
               printExpr (n+1) e

    and spaces n = if n < 0 then raise PrettyPrintNonsense
        else (for i = 1; i <= n; i+1 do print_string " " end)
    and nl() = print_newline()
	
	fun eval(env, pgm) =  // define your eval function
     (Format.printf "********* absyn *********\n";
      printExpr 0 pgm; 
      print_newline())

end

