(* 
   KAIST CS520 2000
   lexer spec
   Kwangkeun Yi
*)   

{
 open Parser
 exception Eof
 exception LexicalError
 fun verbose s = (* (print_string s; print_newline(); s) *) ()
 val comment_depth = ref 0
 val keyword_tbl = Hashtbl.create 31
 val _ = List.iter (fn (keyword, tok) => Hashtbl.add keyword_tbl keyword tok)
                   [
                    ("and", AND),
                    ("or", OR),
                    ("if", IF),
                    ("then",THEN),
                    ("else",ELSE),
                    ("let", LET),
                    ("in", IN),
                    ("end", END),
                    ("fn", FN),
                    ("fix", FX),
                    ("read" , READ),
                    ("write", WRITE),
                    ("val", VAL)
                  ]
     fun s2int "" = raise LexicalError
       | s2int s = if ('~' = String.get s 0) then
                   - (int_of_string(String.sub s 1 ((String.length s)-1)))
                   else int_of_string s
} 

val blank = [' ' '\n' '\t']+
    (* KS X 1001 a.k.a. KS C 5601 or euc-kr *)
val hangul = (['\176'-'\200']['\161'-'\254'])+
    (* KS X 1005-1 a.k.a. KS C 5700, unicode, or ISO/IEC 10646-1 *)
val uhangul = (['\172'-'\214']['\000'-'\255']|'\215'['\000'-'\163'])+
val id = (['a'-'z']|hangul|uhangul)
         (hangul|uhangul|['a'-'z' 'A'-'Z' '\'' '0'-'9' '_'])*
val number = ['0'-'9']+|'~'['0'-'9']+
val charstring = '"' [^ '"' '\n']* '"'

rule start =
 parse blank { start lexbuf }
     | number { NUM (s2int(Lexing.lexeme lexbuf)) }
     | id { let val id = Lexing.lexeme lexbuf
            in Hashtbl.find keyword_tbl id
               handle _ => ID id
            end}
     | charstring { STRING(Lexing.lexeme lexbuf) }         
     | eof { verbose "eof"; EOF}
     | "(*" { comment_depth :=1;
              comment lexbuf;
              start lexbuf }
     | "=>" { verbose "=>"; RARROW}
     | "=" {verbose "="; EQUAL}
     | "+" {verbose "+"; PLUS}
     | "-" {verbose "-"; MINUS}
     | "*" {verbose "+"; STAR}
     | "/" {verbose "-"; SLASH}
     | "<" {verbose "-"; LANGLE}
     | ">" {verbose "-"; RANGLE}
     | "(" { verbose "("; LP}
     | ")" { verbose ")"; RP}
     | _ {raise LexicalError}

and comment = parse
     "(*" {comment_depth := !comment_depth+1; comment lexbuf}
   | "*)" {comment_depth := !comment_depth-1;
           if !comment_depth > 0 then comment lexbuf }
   | eof {raise Eof}
   | _   {comment lexbuf}
